@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2012 - Mauro Carvalho Chehab <mchehab@redhat.com>
+ * Copyright (c) 2011-2012 - Mauro Carvalho Chehab
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,13 @@
 
 struct dvb_v5_fe_parms;
 
+void iconv_to_charset(struct dvb_v5_fe_parms *parms,
+		      char *dest,
+		      size_t destlen,
+		      const unsigned char *src,
+		      size_t len,
+		      char *type, char *output_charset);
+
 void parse_string(struct dvb_v5_fe_parms *parms, char **dest, char **emph,
 		  const unsigned char *src, size_t len,
 		  char *default_charset, char *output_charset);
