@@ -269,7 +269,7 @@ struct parse_event key_events[] = {
 	{"KEY_BRIGHTNESS_CYCLE", 243},
 	{"KEY_BRIGHTNESS_ZERO", 244},
 	{"KEY_DISPLAY_OFF", 245},
-	{"KEY_WIMAX", 246},
+	{"KEY_WWAN", 246},
 	{"KEY_RFKILL", 247},
 	{"KEY_MICMUTE", 248},
 	{"BTN_MISC", 0x100},
@@ -498,6 +498,7 @@ struct parse_event key_events[] = {
 	{"BTN_DPAD_DOWN", 0x221},
 	{"BTN_DPAD_LEFT", 0x222},
 	{"BTN_DPAD_RIGHT", 0x223},
+	{"KEY_ALS_TOGGLE", 0x230},
 	{"BTN_TRIGGER_HAPPY", 0x2c0},
 	{"BTN_TRIGGER_HAPPY1", 0x2c0},
 	{"BTN_TRIGGER_HAPPY2", 0x2c1},
