@@ -21,6 +21,14 @@
 #include "general-tab.h"
 #include "vbi-tab.h"
 #include "capture-win.h"
+#include "capture-win-qt.h"
+#include "capture-win-gl.h"
+
+#ifdef ENABLE_ASLA
+extern "C" {
+#include "alsa_stream.h"
+}
+#endif
 
 #include <QToolBar>
 #include <QToolButton>
@@ -43,15 +51,18 @@
 #include <QWhatsThis>
 #include <QThread>
 #include <QCloseEvent>
+#include <QInputDialog>
 
 #include <assert.h>
 #include <sys/mman.h>
+#include <sys/time.h>
 #include <errno.h>
 #include <dirent.h>
 #include <libv4l2.h>
 
 ApplicationWindow::ApplicationWindow() :
 	m_capture(NULL),
+	m_genTab(NULL),
 	m_sigMapper(NULL)
 {
 	setAttribute(Qt::WA_DeleteOnClose, true);
@@ -74,10 +85,11 @@ ApplicationWindow::ApplicationWindow() :
 	openRawAct->setShortcut(Qt::CTRL+Qt::Key_R);
 	connect(openRawAct, SIGNAL(triggered()), this, SLOT(openrawdev()));
 
-	m_capStartAct = new QAction(QIcon(":/record.png"), "&Start Capturing", this);
+	m_capStartAct = new QAction(QIcon(":/record.png"), "Start &Capturing", this);
 	m_capStartAct->setStatusTip("Start capturing");
 	m_capStartAct->setCheckable(true);
 	m_capStartAct->setDisabled(true);
+	m_capStartAct->setShortcut(Qt::CTRL+Qt::Key_V);
 	connect(m_capStartAct, SIGNAL(toggled(bool)), this, SLOT(capStart(bool)));
 
 	m_snapshotAct = new QAction(QIcon(":/snapshot.png"), "&Make Snapshot", this);
@@ -91,7 +103,7 @@ ApplicationWindow::ApplicationWindow() :
 	m_saveRawAct->setChecked(false);
 	connect(m_saveRawAct, SIGNAL(toggled(bool)), this, SLOT(saveRaw(bool)));
 
-	m_showFramesAct = new QAction(QIcon(":/video-television.png"), "Show &Frames", this);
+	m_showFramesAct = new QAction(QIcon(":/video-television.png"), "&Show Frames", this);
 	m_showFramesAct->setStatusTip("Only show captured frames if set.");
 	m_showFramesAct->setCheckable(true);
 	m_showFramesAct->setChecked(true);
@@ -110,10 +122,8 @@ ApplicationWindow::ApplicationWindow() :
 	fileMenu->addAction(openAct);
 	fileMenu->addAction(openRawAct);
 	fileMenu->addAction(closeAct);
-	fileMenu->addAction(m_capStartAct);
 	fileMenu->addAction(m_snapshotAct);
 	fileMenu->addAction(m_saveRawAct);
-	fileMenu->addAction(m_showFramesAct);
 	fileMenu->addSeparator();
 	fileMenu->addAction(quitAct);
 
@@ -127,6 +137,49 @@ ApplicationWindow::ApplicationWindow() :
 	toolBar->addSeparator();
 	toolBar->addAction(quitAct);
 
+	m_scalingAct = new QAction("&Enable Video Scaling", this);
+	m_scalingAct->setStatusTip("Scale video frames to match window size if set");
+	m_scalingAct->setCheckable(true);
+	m_scalingAct->setChecked(true);
+	connect(m_scalingAct, SIGNAL(toggled(bool)), this, SLOT(enableScaling(bool)));
+	m_resetScalingAct = new QAction("Resize to &Frame Size", this);
+	m_resetScalingAct->setStatusTip("Resizes the capture window to match frame size");
+	m_resetScalingAct->setShortcut(Qt::CTRL+Qt::Key_F);
+
+	QMenu *captureMenu = menuBar()->addMenu("&Capture");
+	captureMenu->addAction(m_capStartAct);
+	captureMenu->addAction(m_showFramesAct);
+	captureMenu->addAction(m_scalingAct);
+	captureMenu->addAction(m_resetScalingAct);
+
+	if (CaptureWinGL::isSupported()) {
+		m_renderMethod = QV4L2_RENDER_GL;
+
+		m_useGLAct = new QAction("Use Open&GL Rendering", this);
+		m_useGLAct->setStatusTip("Use GPU with OpenGL for video capture if set.");
+		m_useGLAct->setCheckable(true);
+		m_useGLAct->setChecked(true);
+		connect(m_useGLAct, SIGNAL(triggered()), this, SLOT(setRenderMethod()));
+		captureMenu->addAction(m_useGLAct);
+	} else {
+		m_renderMethod = QV4L2_RENDER_QT;
+	}
+
+#ifdef HAVE_ALSA
+	captureMenu->addSeparator();
+
+	m_showAllAudioAct = new QAction("Show All Audio &Devices", this);
+	m_showAllAudioAct->setStatusTip("Show all audio input and output devices if set");
+	m_showAllAudioAct->setCheckable(true);
+	m_showAllAudioAct->setChecked(false);
+	captureMenu->addAction(m_showAllAudioAct);
+
+	m_audioBufferAct = new QAction("Set Audio &Buffer Capacity...", this);
+	m_audioBufferAct->setStatusTip("Set audio buffer capacity in amount of ms than can be stored");
+	connect(m_audioBufferAct, SIGNAL(triggered()), this, SLOT(setAudioBufferSize()));
+	captureMenu->addAction(m_audioBufferAct);
+#endif
+
 	QMenu *helpMenu = menuBar()->addMenu("&Help");
 	helpMenu->addAction("&About", this, SLOT(about()), Qt::Key_F1);
 
@@ -154,15 +207,32 @@ void ApplicationWindow::setDevice(const QString &device, bool rawOpen)
 	m_sigMapper = new QSignalMapper(this);
 	connect(m_sigMapper, SIGNAL(mapped(int)), this, SLOT(ctrlAction(int)));
 
-	if (!open(device, !rawOpen))
+	if (!open(device, !rawOpen)) {
+#ifdef HAVE_ALSA
+		m_showAllAudioAct->setEnabled(false);
+		m_audioBufferAct->setEnabled(false);
+#endif
 		return;
+	}
 
-	m_capture = new CaptureWin;
-	m_capture->setMinimumSize(150, 50);
-	connect(m_capture, SIGNAL(close()), this, SLOT(closeCaptureWin()));
+	newCaptureWin();
 
 	QWidget *w = new QWidget(m_tabs);
 	m_genTab = new GeneralTab(device, *this, 4, w);
+
+#ifdef HAVE_ALSA
+	if (m_genTab->hasAlsaAudio()) {
+		connect(m_showAllAudioAct, SIGNAL(toggled(bool)), m_genTab, SLOT(showAllAudioDevices(bool)));
+		connect(m_genTab, SIGNAL(audioDeviceChanged()), this, SLOT(changeAudioDevice()));
+		m_showAllAudioAct->setEnabled(true);
+		m_audioBufferAct->setEnabled(true);
+	} else {
+		m_showAllAudioAct->setEnabled(false);
+		m_audioBufferAct->setEnabled(false);
+	}
+#endif
+	connect(m_genTab, SIGNAL(pixelAspectRatioChanged()), this, SLOT(updatePixelAspectRatio()));
+	connect(m_genTab, SIGNAL(cropChanged()), this, SLOT(updateCropping()));
 	m_tabs->addTab(w, "General");
 	addTabs();
 	if (caps() & (V4L2_CAP_VBI_CAPTURE | V4L2_CAP_SLICED_VBI_CAPTURE)) {
@@ -177,14 +247,14 @@ void ApplicationWindow::setDevice(const QString &device, bool rawOpen)
 	m_tabs->show();
 	m_tabs->setFocus();
 	m_convertData = v4lconvert_create(fd());
-	m_capStartAct->setEnabled(fd() >= 0 && !m_genTab->isRadio());
+	m_capStartAct->setEnabled(fd() >= 0);
 	m_ctrlNotifier = new QSocketNotifier(fd(), QSocketNotifier::Exception, m_tabs);
 	connect(m_ctrlNotifier, SIGNAL(activated(int)), this, SLOT(ctrlEvent()));
 }
 
 void ApplicationWindow::opendev()
 {
-	QFileDialog d(this, "Select v4l device", "/dev", "V4L Devices (video* vbi* radio*)");
+	QFileDialog d(this, "Select v4l device", "/dev", "V4L Devices (video* vbi* radio* swradio*)");
 
 	d.setFilter(QDir::AllDirs | QDir::Files | QDir::System);
 	d.setFileMode(QFileDialog::ExistingFile);
@@ -194,7 +264,7 @@ void ApplicationWindow::opendev()
 
 void ApplicationWindow::openrawdev()
 {
-	QFileDialog d(this, "Select v4l device", "/dev", "V4L Devices (video* vbi* radio*)");
+	QFileDialog d(this, "Select v4l device", "/dev", "V4L Devices (video* vbi* radio* swradio*)");
 
 	d.setFilter(QDir::AllDirs | QDir::Files | QDir::System);
 	d.setFileMode(QFileDialog::ExistingFile);
@@ -202,6 +272,107 @@ void ApplicationWindow::openrawdev()
 		setDevice(d.selectedFiles().first(), true);
 }
 
+void ApplicationWindow::setRenderMethod()
+{
+	if (m_capStartAct->isChecked()) {
+		m_useGLAct->setChecked(m_renderMethod == QV4L2_RENDER_GL);
+		return;
+	}
+
+	if (m_useGLAct->isChecked())
+		m_renderMethod = QV4L2_RENDER_GL;
+	else
+		m_renderMethod = QV4L2_RENDER_QT;
+
+	newCaptureWin();
+}
+
+void ApplicationWindow::setAudioBufferSize()
+{
+	bool ok;
+	int buffer = QInputDialog::getInt(this, "Audio Device Buffer Size", "Capacity in ms:",
+					   m_genTab->getAudioDeviceBufferSize(), 1, 65535, 1, &ok);
+
+	if (ok) {
+		m_genTab->setAudioDeviceBufferSize(buffer);
+		changeAudioDevice();
+	}
+}
+
+
+void ApplicationWindow::ctrlEvent()
+{
+	v4l2_event ev;
+
+	while (dqevent(ev)) {
+		if (ev.type == V4L2_EVENT_SOURCE_CHANGE) {
+			m_genTab->sourceChange(ev);
+			continue;
+		}
+		if (ev.type != V4L2_EVENT_CTRL)
+			continue;
+		m_ctrlMap[ev.id].flags = ev.u.ctrl.flags;
+		m_ctrlMap[ev.id].minimum = ev.u.ctrl.minimum;
+		m_ctrlMap[ev.id].maximum = ev.u.ctrl.maximum;
+		m_ctrlMap[ev.id].step = ev.u.ctrl.step;
+		m_ctrlMap[ev.id].default_value = ev.u.ctrl.default_value;
+		m_widgetMap[ev.id]->setDisabled(m_ctrlMap[ev.id].flags & CTRL_FLAG_DISABLED);
+		switch (m_ctrlMap[ev.id].type) {
+		case V4L2_CTRL_TYPE_INTEGER:
+		case V4L2_CTRL_TYPE_INTEGER_MENU:
+		case V4L2_CTRL_TYPE_MENU:
+		case V4L2_CTRL_TYPE_BOOLEAN:
+		case V4L2_CTRL_TYPE_BITMASK:
+			setVal(ev.id, ev.u.ctrl.value);
+			break;
+		case V4L2_CTRL_TYPE_INTEGER64:
+			setVal64(ev.id, ev.u.ctrl.value64);
+			break;
+		default:
+			break;
+		}
+		if (m_ctrlMap[ev.id].type != V4L2_CTRL_TYPE_STRING)
+			continue;
+		queryctrl(m_ctrlMap[ev.id]);
+
+		struct v4l2_ext_control c;
+		struct v4l2_ext_controls ctrls;
+
+		c.id = ev.id;
+		c.size = m_ctrlMap[ev.id].maximum + 1;
+		c.string = (char *)malloc(c.size);
+		memset(&ctrls, 0, sizeof(ctrls));
+		ctrls.count = 1;
+		ctrls.ctrl_class = 0;
+		ctrls.controls = &c;
+		if (!ioctl(VIDIOC_G_EXT_CTRLS, &ctrls))
+			setString(ev.id, c.string);
+		free(c.string);
+	}
+}
+
+void ApplicationWindow::newCaptureWin()
+{
+	if (m_capture != NULL) {
+		m_capture->stop();
+		delete m_capture;
+	}
+
+	switch (m_renderMethod) {
+	case QV4L2_RENDER_GL:
+		m_capture = new CaptureWinGL;
+		break;
+	default:
+		m_capture = new CaptureWinQt;
+		break;
+	}
+
+	m_capture->setPixelAspectRatio(1.0);
+	m_capture->enableScaling(m_scalingAct->isChecked());
+        connect(m_capture, SIGNAL(close()), this, SLOT(closeCaptureWin()));
+	connect(m_resetScalingAct, SIGNAL(triggered()), m_capture, SLOT(resetSize()));
+}
+
 void ApplicationWindow::capVbiFrame()
 {
 	__u32 buftype = m_genTab->bufType();
@@ -231,7 +402,11 @@ void ApplicationWindow::capVbiFrame()
 		}
 		if (again)
 			return;
-		data = (__u8 *)m_buffers[buf.index].start;
+		if (buf.flags & V4L2_BUF_FLAG_ERROR) {
+			qbuf(buf);
+			return;
+		}
+		data = (__u8 *)m_buffers[buf.index].start[0];
 		s = buf.bytesused;
 		break;
 
@@ -243,6 +418,10 @@ void ApplicationWindow::capVbiFrame()
 		}
 		if (again)
 			return;
+		if (buf.flags & V4L2_BUF_FLAG_ERROR) {
+			qbuf(buf);
+			return;
+		}
 		data = (__u8 *)buf.m.userptr;
 		s = buf.bytesused;
 		break;
@@ -252,7 +431,7 @@ void ApplicationWindow::capVbiFrame()
 		m_capStartAct->setChecked(false);
 		return;
 	}
-	if (m_showFrames) {
+	if (showFrames()) {
 		for (unsigned y = 0; y < m_vbiHeight; y++) {
 			__u8 *p = data + y * m_vbiWidth;
 			__u8 *q = m_capImage->bits() + y * m_capImage->bytesPerLine();
@@ -296,8 +475,10 @@ void ApplicationWindow::capVbiFrame()
 		m_tv = tv;
 	}
 	status = QString("Frame: %1 Fps: %2").arg(++m_frame).arg(m_fps);
-	if (m_showFrames)
-		m_capture->setImage(*m_capImage, status);
+	if (showFrames())
+		m_capture->setFrame(m_capImage->width(), m_capImage->height(),
+				    m_capDestFormat.fmt.pix.pixelformat, m_capImage->bits(), NULL, status);
+
 	curStatus = statusBar()->currentMessage();
 	if (curStatus.isEmpty() || curStatus.startsWith("Frame: "))
 		statusBar()->showMessage(status);
@@ -305,64 +486,28 @@ void ApplicationWindow::capVbiFrame()
 		refresh();
 }
 
-void ApplicationWindow::ctrlEvent()
-{
-	v4l2_event ev;
-
-	while (dqevent(ev)) {
-		if (ev.type != V4L2_EVENT_CTRL)
-			continue;
-		m_ctrlMap[ev.id].flags = ev.u.ctrl.flags;
-		m_ctrlMap[ev.id].minimum = ev.u.ctrl.minimum;
-		m_ctrlMap[ev.id].maximum = ev.u.ctrl.maximum;
-		m_ctrlMap[ev.id].step = ev.u.ctrl.step;
-		m_ctrlMap[ev.id].default_value = ev.u.ctrl.default_value;
-		m_widgetMap[ev.id]->setDisabled(m_ctrlMap[ev.id].flags & CTRL_FLAG_DISABLED);
-		switch (m_ctrlMap[ev.id].type) {
-		case V4L2_CTRL_TYPE_INTEGER:
-		case V4L2_CTRL_TYPE_INTEGER_MENU:
-		case V4L2_CTRL_TYPE_MENU:
-		case V4L2_CTRL_TYPE_BOOLEAN:
-		case V4L2_CTRL_TYPE_BITMASK:
-			setVal(ev.id, ev.u.ctrl.value);
-			break;
-		case V4L2_CTRL_TYPE_INTEGER64:
-			setVal64(ev.id, ev.u.ctrl.value64);
-			break;
-		default:
-			break;
-		}
-		if (m_ctrlMap[ev.id].type != V4L2_CTRL_TYPE_STRING)
-			continue;
-		queryctrl(m_ctrlMap[ev.id]);
-
-		struct v4l2_ext_control c;
-		struct v4l2_ext_controls ctrls;
-
-		c.id = ev.id;
-		c.size = m_ctrlMap[ev.id].maximum + 1;
-		c.string = (char *)malloc(c.size);
-		memset(&ctrls, 0, sizeof(ctrls));
-		ctrls.count = 1;
-		ctrls.ctrl_class = 0;
-		ctrls.controls = &c;
-		if (!ioctl(VIDIOC_G_EXT_CTRLS, &ctrls))
-			setString(ev.id, c.string);
-		free(c.string);
-	}
-}
-
 void ApplicationWindow::capFrame()
 {
 	__u32 buftype = m_genTab->bufType();
+	v4l2_plane planes[VIDEO_MAX_PLANES];
 	v4l2_buffer buf;
 	int s = 0;
 	int err = 0;
 	bool again;
+#ifdef HAVE_ALSA
+	struct timeval tv_alsa;
+#endif
+
+	unsigned char *displaybuf = NULL;
+	unsigned char *displaybuf2 = NULL;
 
 	switch (m_capMethod) {
 	case methodRead:
 		s = read(m_frameData, m_capSrcFormat.fmt.pix.sizeimage);
+#ifdef HAVE_ALSA
+		alsa_thread_timestamp(&tv_alsa);
+#endif
+
 		if (s < 0) {
 			if (errno != EAGAIN) {
 				error("read");
@@ -375,69 +520,106 @@ void ApplicationWindow::capFrame()
 		if (m_saveRaw.openMode())
 			m_saveRaw.write((const char *)m_frameData, s);
 
-		if (!m_showFrames)
+		if (!showFrames())
 			break;
 		if (m_mustConvert)
 			err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
-				m_frameData, s,
-				m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
-		if (!m_mustConvert || err < 0)
-			memcpy(m_capImage->bits(), m_frameData, std::min(s, m_capImage->numBytes()));
+						 m_frameData, s,
+						 m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+		if (m_mustConvert && err != -1)
+			displaybuf = m_capImage->bits();
+		if (!m_mustConvert)
+			displaybuf = m_frameData;
 		break;
 
 	case methodMmap:
+		memset(planes, 0, sizeof(planes));
+		buf.length = VIDEO_MAX_PLANES;
+		buf.m.planes = planes;
 		if (!dqbuf_mmap(buf, buftype, again)) {
 			error("dqbuf");
 			m_capStartAct->setChecked(false);
 			return;
 		}
+
 		if (again)
 			return;
+		if (buf.flags & V4L2_BUF_FLAG_ERROR) {
+			qbuf(buf);
+			return;
+		}
 
-		if (m_showFrames) {
+#ifdef HAVE_ALSA
+		alsa_thread_timestamp(&tv_alsa);
+#endif
+
+		if (showFrames()) {
 			if (m_mustConvert)
-				err = v4lconvert_convert(m_convertData,
-					&m_capSrcFormat, &m_capDestFormat,
-					(unsigned char *)m_buffers[buf.index].start, buf.bytesused,
-					m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
-			if (!m_mustConvert || err < 0)
-				memcpy(m_capImage->bits(),
-				       (unsigned char *)m_buffers[buf.index].start,
-				       std::min(buf.bytesused, (unsigned)m_capImage->numBytes()));
+				err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
+							 (unsigned char *)m_buffers[buf.index].start[0], buf.bytesused,
+							 m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+			if (m_mustConvert && err != -1)
+				displaybuf = m_capImage->bits();
+			if (!m_mustConvert) {
+				displaybuf = (unsigned char *)m_buffers[buf.index].start[0];
+				displaybuf2 = (unsigned char *)m_buffers[buf.index].start[1];
+				if (V4L2_TYPE_IS_MULTIPLANAR(buf.type)) {
+					displaybuf += planes[0].data_offset;
+					displaybuf2 += planes[1].data_offset;
+				}
+			}
 		}
 		if (m_makeSnapshot)
-			makeSnapshot((unsigned char *)m_buffers[buf.index].start, buf.bytesused);
+			makeSnapshot((unsigned char *)m_buffers[buf.index].start[0], buf.bytesused);
 		if (m_saveRaw.openMode())
-			m_saveRaw.write((const char *)m_buffers[buf.index].start, buf.bytesused);
+			m_saveRaw.write((const char *)m_buffers[buf.index].start[0], buf.bytesused);
 
-		qbuf(buf);
 		break;
 
 	case methodUser:
+		memset(planes, 0, sizeof(planes));
+		buf.length = VIDEO_MAX_PLANES;
+		buf.m.planes = planes;
 		if (!dqbuf_user(buf, buftype, again)) {
-			error("dqbuf");
+			error("dqbuf1");
 			m_capStartAct->setChecked(false);
 			return;
 		}
 		if (again)
 			return;
+		if (buf.flags & V4L2_BUF_FLAG_ERROR) {
+			qbuf(buf);
+			return;
+		}
+#ifdef HAVE_ALSA
+		alsa_thread_timestamp(&tv_alsa);
+#endif
+		if (buf.flags & V4L2_BUF_FLAG_ERROR) {
+			qbuf(buf);
+			return;
+		}
 
-		if (m_showFrames) {
+		if (showFrames()) {
 			if (m_mustConvert)
-				err = v4lconvert_convert(m_convertData,
-					&m_capSrcFormat, &m_capDestFormat,
-					(unsigned char *)buf.m.userptr, buf.bytesused,
-					m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
-			if (!m_mustConvert || err < 0)
-				memcpy(m_capImage->bits(), (unsigned char *)buf.m.userptr,
-				       std::min(buf.bytesused, (unsigned)m_capImage->numBytes()));
+				err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
+							 (unsigned char *)buf.m.userptr, buf.bytesused,
+							 m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+			if (m_mustConvert && err != -1)
+				displaybuf = m_capImage->bits();
+			if (!m_mustConvert) {
+				displaybuf = (unsigned char *)m_buffers[buf.index].start[0];
+				displaybuf2 = (unsigned char *)m_buffers[buf.index].start[1];
+				if (V4L2_TYPE_IS_MULTIPLANAR(buf.type)) {
+					displaybuf += planes[0].data_offset;
+					displaybuf2 += planes[1].data_offset;
+				}
+			}
 		}
 		if (m_makeSnapshot)
 			makeSnapshot((unsigned char *)buf.m.userptr, buf.bytesused);
 		if (m_saveRaw.openMode())
 			m_saveRaw.write((const char *)buf.m.userptr, buf.bytesused);
 
-		qbuf(buf);
 		break;
 	}
 	if (err == -1 && m_frame == 0)
@@ -456,9 +638,29 @@ void ApplicationWindow::capFrame()
 		m_lastFrame = m_frame;
 		m_tv = tv;
 	}
+
+
 	status = QString("Frame: %1 Fps: %2").arg(++m_frame).arg(m_fps);
-	if (m_showFrames)
-		m_capture->setImage(*m_capImage, status);
+#ifdef HAVE_ALSA
+	if (alsa_thread_is_running()) {
+		if (tv_alsa.tv_sec || tv_alsa.tv_usec) {
+			m_totalAudioLatency.tv_sec += buf.timestamp.tv_sec - tv_alsa.tv_sec;
+			m_totalAudioLatency.tv_usec += buf.timestamp.tv_usec - tv_alsa.tv_usec;
+		}
+		status.append(QString(" Average A-V: %3 ms")
+			      .arg((m_totalAudioLatency.tv_sec * 1000 + m_totalAudioLatency.tv_usec / 1000) / m_frame));
+	}
+#endif
+	if (displaybuf == NULL && showFrames())
+		status.append(" Error: Unsupported format.");
+
+	if (showFrames())
+		m_capture->setFrame(m_capImage->width(), m_capImage->height(),
+				    m_capDestFormat.fmt.pix.pixelformat, displaybuf, displaybuf2, status);
+
+	if (m_capMethod == methodMmap || m_capMethod == methodUser)
+		qbuf(buf);
+
 	curStatus = statusBar()->currentMessage();
 	if (curStatus.isEmpty() || curStatus.startsWith("Frame: "))
 		statusBar()->showMessage(status);
@@ -468,6 +670,11 @@ void ApplicationWindow::capFrame()
 
 bool ApplicationWindow::startCapture(unsigned buffer_size)
 {
+	startAudio();
+
+	if (m_genTab->isRadio())
+		return true;
+
 	__u32 buftype = m_genTab->bufType();
 	v4l2_requestbuffers req;
 	unsigned int i;
@@ -477,6 +684,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 	switch (m_capMethod) {
 	case methodRead:
 		m_snapshotAct->setEnabled(true);
+		m_genTab->setHaveBuffers(true);
 		/* Nothing to do. */
 		return true;
 
@@ -501,6 +709,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 		}
 
 		for (m_nbuffers = 0; m_nbuffers < req.count; ++m_nbuffers) {
+			v4l2_plane planes[VIDEO_MAX_PLANES];
 			v4l2_buffer buf;
 
 			memset(&buf, 0, sizeof(buf));
@@ -508,18 +717,32 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 			buf.type        = buftype;
 			buf.memory      = V4L2_MEMORY_MMAP;
 			buf.index       = m_nbuffers;
+			buf.length      = VIDEO_MAX_PLANES;
+			buf.m.planes    = planes;
 
 			if (-1 == ioctl(VIDIOC_QUERYBUF, &buf)) {
 				perror("VIDIOC_QUERYBUF");
 				goto error;
 			}
 
-			m_buffers[m_nbuffers].length = buf.length;
-			m_buffers[m_nbuffers].start = mmap(buf.length, buf.m.offset);
-
-			if (MAP_FAILED == m_buffers[m_nbuffers].start) {
-				perror("mmap");
-				goto error;
+			if (V4L2_TYPE_IS_MULTIPLANAR(buftype)) {
+				m_buffers[m_nbuffers].planes = buf.length;
+				for (unsigned p = 0; p < buf.length; p++) {
+					m_buffers[m_nbuffers].length[p] = planes[p].length;
+					m_buffers[m_nbuffers].start[p] = mmap(planes[p].length, planes[p].m.mem_offset);
+					if (MAP_FAILED == m_buffers[m_nbuffers].start[p]) {
+						perror("mmap");
+						goto error;
+					}
+				}
+			} else {
+				m_buffers[m_nbuffers].planes = 1;
+				m_buffers[m_nbuffers].length[0] = buf.length;
+				m_buffers[m_nbuffers].start[0] = mmap(buf.length, buf.m.offset);
+				if (MAP_FAILED == m_buffers[m_nbuffers].start[0]) {
+					perror("mmap");
+					goto error;
+				}
 			}
 		}
 		for (i = 0; i < m_nbuffers; ++i) {
@@ -533,6 +756,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 			goto error;
 		}
 		m_snapshotAct->setEnabled(true);
+		m_genTab->setHaveBuffers(true);
 		return true;
 
 	case methodUser:
@@ -555,13 +779,42 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 		}
 
 		for (m_nbuffers = 0; m_nbuffers < req.count; ++m_nbuffers) {
-			m_buffers[m_nbuffers].length = buffer_size;
-			m_buffers[m_nbuffers].start = malloc(buffer_size);
+			v4l2_plane planes[VIDEO_MAX_PLANES];
+			v4l2_buffer buf;
 
-			if (!m_buffers[m_nbuffers].start) {
-				error("Out of memory");
+			memset(&buf, 0, sizeof(buf));
+
+			buf.type        = buftype;
+			buf.memory      = V4L2_MEMORY_USERPTR;
+			buf.index       = m_nbuffers;
+			buf.length      = VIDEO_MAX_PLANES;
+			buf.m.planes    = planes;
+
+			if (-1 == ioctl(VIDIOC_QUERYBUF, &buf)) {
+				perror("VIDIOC_QUERYBUF");
 				goto error;
 			}
+
+			if (V4L2_TYPE_IS_MULTIPLANAR(buftype)) {
+				m_buffers[m_nbuffers].planes = buf.length;
+				for (unsigned p = 0; p < buf.length; p++) {
+					m_buffers[m_nbuffers].length[p] = planes[p].length;
+					m_buffers[m_nbuffers].start[p] = malloc(planes[p].length);
+					if (m_buffers[m_nbuffers].start[p] == NULL) {
+						error("Out of memory");
+						goto error;
+					}
+				}
+			} else {
+				m_buffers[m_nbuffers].planes = 1;
+				m_buffers[m_nbuffers].length[0] = buffer_size;
+				m_buffers[m_nbuffers].start[0] = malloc(buffer_size);
+
+				if (!m_buffers[m_nbuffers].start[0]) {
+					error("Out of memory");
+					goto error;
+				}
+			}
 		}
 		for (i = 0; i < m_nbuffers; ++i)
 			if (!qbuf_user(i, buftype, m_buffers[i].start, m_buffers[i].length)) {
@@ -573,6 +826,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 			goto error;
 		}
 		m_snapshotAct->setEnabled(true);
+		m_genTab->setHaveBuffers(true);
 		return true;
 	}
 
@@ -583,11 +837,17 @@ error:
 
 void ApplicationWindow::stopCapture()
 {
+	stopAudio();
+
+	if (m_genTab->isRadio())
+		return;
+
 	__u32 buftype = m_genTab->bufType();
 	v4l2_requestbuffers reqbufs;
 	v4l2_encoder_cmd cmd;
 	unsigned i;
 
+	m_capture->stop();
 	m_snapshotAct->setDisabled(true);
 	switch (m_capMethod) {
 	case methodRead:
@@ -602,8 +862,9 @@ void ApplicationWindow::stopCapture()
 		if (!streamoff(buftype))
 			perror("VIDIOC_STREAMOFF");
 		for (i = 0; i < m_nbuffers; ++i)
-			if (-1 == munmap(m_buffers[i].start, m_buffers[i].length))
-				perror("munmap");
+			for (unsigned p = 0; p < m_buffers[i].planes; p++)
+				if (-1 == munmap(m_buffers[i].start[p], m_buffers[i].length[p]))
+					perror("munmap");
 		// Free all buffers.
 		reqbufs_mmap(reqbufs, buftype, 1);  // videobuf workaround
 		reqbufs_mmap(reqbufs, buftype, 0);
@@ -616,14 +877,25 @@ void ApplicationWindow::stopCapture()
 		reqbufs_user(reqbufs, buftype, 1);  // videobuf workaround
 		reqbufs_user(reqbufs, buftype, 0);
 		for (i = 0; i < m_nbuffers; ++i)
-			free(m_buffers[i].start);
+			for (unsigned p = 0; p < m_buffers[i].planes; p++)
+				free(m_buffers[i].start[p]);
 		break;
 	}
 	free(m_buffers);
 	m_buffers = NULL;
+	m_genTab->setHaveBuffers(false);
 	refresh();
 }
 
+bool ApplicationWindow::showFrames()
+{
+	if (m_showFramesAct->isChecked() && !m_capture->isVisible())
+		m_capture->show();
+	if (!m_showFramesAct->isChecked() && m_capture->isVisible())
+		m_capture->hide();
+	return m_showFramesAct->isChecked();
+}
+
 void ApplicationWindow::startOutput(unsigned)
 {
 }
@@ -632,6 +904,60 @@ void ApplicationWindow::stopOutput()
 {
 }
 
+void ApplicationWindow::enableScaling(bool enable)
+{
+	if (m_capture != NULL)
+		m_capture->enableScaling(enable);
+}
+
+void ApplicationWindow::updatePixelAspectRatio()
+{
+	if (m_capture != NULL && m_genTab != NULL)
+		m_capture->setPixelAspectRatio(m_genTab->getPixelAspectRatio());
+}
+
+void ApplicationWindow::updateCropping()
+{
+	if (m_capture != NULL)
+		m_capture->setCropMethod(m_genTab->getCropMethod());
+}
+
+void ApplicationWindow::startAudio()
+{
+#ifdef HAVE_ALSA
+	m_totalAudioLatency.tv_sec = 0;
+	m_totalAudioLatency.tv_usec = 0;
+
+	QString audIn = m_genTab->getAudioInDevice();
+	QString audOut = m_genTab->getAudioOutDevice();
+
+	if (audIn != NULL && audOut != NULL && audIn.compare("None") && audIn.compare(audOut) != 0) {
+		alsa_thread_startup(audOut.toAscii().data(), audIn.toAscii().data(),
+				    m_genTab->getAudioDeviceBufferSize(), NULL, 0);
+
+		if (m_genTab->isRadio())
+			statusBar()->showMessage("Capturing audio");
+	}
+#endif
+}
+
+void ApplicationWindow::stopAudio()
+{
+#ifdef HAVE_ALSA
+	if (m_genTab != NULL && m_genTab->isRadio())
+		statusBar()->showMessage("");
+	alsa_thread_stop();
+#endif
+}
+
+void ApplicationWindow::changeAudioDevice()
+{
+	stopAudio();
+	if (m_capStartAct->isChecked()) {
+		startAudio();
+	}
+}
+
 void ApplicationWindow::closeCaptureWin()
 {
 	m_capStartAct->setChecked(false);
@@ -639,28 +965,24 @@ void ApplicationWindow::closeCaptureWin()
 
 void ApplicationWindow::capStart(bool start)
 {
-	static const struct {
-		__u32 v4l2_pixfmt;
-		QImage::Format qt_pixfmt;
-	} supported_fmts[] = {
-#if Q_BYTE_ORDER == Q_BIG_ENDIAN
-		{ V4L2_PIX_FMT_RGB32, QImage::Format_RGB32 },
-		{ V4L2_PIX_FMT_RGB24, QImage::Format_RGB888 },
-		{ V4L2_PIX_FMT_RGB565X, QImage::Format_RGB16 },
-		{ V4L2_PIX_FMT_RGB555X, QImage::Format_RGB555 },
-#else
-		{ V4L2_PIX_FMT_BGR32, QImage::Format_RGB32 },
-		{ V4L2_PIX_FMT_RGB24, QImage::Format_RGB888 },
-		{ V4L2_PIX_FMT_RGB565, QImage::Format_RGB16 },
-		{ V4L2_PIX_FMT_RGB555, QImage::Format_RGB555 },
-		{ V4L2_PIX_FMT_RGB444, QImage::Format_RGB444 },
-#endif
-		{ 0, QImage::Format_Invalid }
-	};
+	if (m_genTab->isRadio()) {
+		if (start)
+			startCapture(0);
+		else
+			stopCapture();
+
+		return;
+	}
+
 	QImage::Format dstFmt = QImage::Format_RGB888;
 	struct v4l2_fract interval;
 	v4l2_pix_format &srcPix = m_capSrcFormat.fmt.pix;
 	v4l2_pix_format &dstPix = m_capDestFormat.fmt.pix;
+	v4l2_pix_format_mplane &srcMPix = m_capSrcFormat.fmt.pix_mp;
+	v4l2_pix_format_mplane &dstMPix = m_capDestFormat.fmt.pix_mp;
+	__u32 buftype = m_genTab->bufType();
+	bool isPlanar = m_genTab->isPlanar();
+	__u32 width, height, pixfmt;
 
 	if (!start) {
 		stopCapture();
@@ -670,7 +992,6 @@ void ApplicationWindow::capStart(bool start)
 		m_capImage = NULL;
 		return;
 	}
-	m_showFrames = m_showFramesAct->isChecked();
 	m_frame = m_lastFrame = m_fps = 0;
 	m_capMethod = m_genTab->capMethod();
 
@@ -678,7 +999,6 @@ void ApplicationWindow::capStart(bool start)
 		v4l2_format fmt;
 		v4l2_std_id std;
 
-		m_showFrames = false;
 		g_fmt_sliced_vbi(fmt);
 		g_std(std);
 		fmt.fmt.sliced.service_set = (std & V4L2_STD_625_50) ?
@@ -688,6 +1008,7 @@ void ApplicationWindow::capStart(bool start)
 		m_vbiTab->slicedFormat(fmt.fmt.sliced);
 		m_vbiSize = fmt.fmt.sliced.io_size;
 		m_frameData = new unsigned char[m_vbiSize];
+		updatePixelAspectRatio();
 		if (startCapture(m_vbiSize)) {
 			m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
 			connect(m_capNotifier, SIGNAL(activated(int)), this, SLOT(capVbiFrame()));
@@ -717,13 +1038,14 @@ void ApplicationWindow::capStart(bool start)
 			m_vbiHeight = fmt.fmt.vbi.count[0] + fmt.fmt.vbi.count[1];
 		m_vbiSize = m_vbiWidth * m_vbiHeight;
 		m_frameData = new unsigned char[m_vbiSize];
-		if (m_showFrames) {
-			m_capture->setMinimumSize(m_vbiWidth, m_vbiHeight);
-			m_capImage = new QImage(m_vbiWidth, m_vbiHeight, dstFmt);
-			m_capImage->fill(0);
-			m_capture->setImage(*m_capImage, "No frame");
+		m_capImage = new QImage(m_vbiWidth, m_vbiHeight, dstFmt);
+		m_capImage->fill(0);
+		m_capture->setFrame(m_capImage->width(), m_capImage->height(),
+				    m_capDestFormat.fmt.pix.pixelformat, m_capImage->bits(), NULL, "No frame");
+		m_capture->resize(m_vbiWidth, m_vbiHeight);
+		if (showFrames())
 			m_capture->show();
-		}
+
 		statusBar()->showMessage("No frame");
 		if (startCapture(m_vbiSize)) {
 			m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
@@ -732,42 +1054,58 @@ void ApplicationWindow::capStart(bool start)
 		return;
 	}
 
-	g_fmt_cap(m_capSrcFormat);
+	g_fmt_cap(buftype, m_capSrcFormat);
 	s_fmt(m_capSrcFormat);
 	if (m_genTab->get_interval(interval))
-		set_interval(interval);
+		set_interval(buftype, interval);
+
+	if (!isPlanar)
+		m_frameData = new unsigned char[srcPix.sizeimage];
+	else
+		m_frameData = new unsigned char[srcMPix.plane_fmt[0].sizeimage + srcMPix.plane_fmt[1].sizeimage];
+	m_capDestFormat = m_capSrcFormat;
+
+	if (isPlanar) {
+		dstMPix.pixelformat = srcMPix.pixelformat;
+		width = srcMPix.width;
+		height = srcMPix.height;
+		pixfmt = srcMPix.pixelformat;
+		m_mustConvert = false;
+	} else if (m_capture->hasNativeFormat(srcPix.pixelformat)) {
+		dstPix.pixelformat = srcPix.pixelformat;
+		width = srcPix.width;
+		height = srcPix.height;
+		pixfmt = srcPix.pixelformat;
+		m_mustConvert = false;
+	} else {
+		m_mustConvert = true;
 
-	m_mustConvert = m_showFrames;
-	m_frameData = new unsigned char[srcPix.sizeimage];
-	if (m_showFrames) {
-		m_capDestFormat = m_capSrcFormat;
 		dstPix.pixelformat = V4L2_PIX_FMT_RGB24;
-
-		for (int i = 0; supported_fmts[i].v4l2_pixfmt; i++) {
-			if (supported_fmts[i].v4l2_pixfmt == srcPix.pixelformat) {
-				dstPix.pixelformat = supported_fmts[i].v4l2_pixfmt;
-				dstFmt = supported_fmts[i].qt_pixfmt;
-				m_mustConvert = false;
-				break;
-			}
-		}
-		if (m_mustConvert) {
-			v4l2_format copy = m_capSrcFormat;
-
-			v4lconvert_try_format(m_convertData, &m_capDestFormat, &m_capSrcFormat);
-			// v4lconvert_try_format sometimes modifies the source format if it thinks
-			// that there is a better format available. Restore our selected source
-			// format since we do not want that happening.
-			m_capSrcFormat = copy;
-		}
-
-		m_capture->setMinimumSize(dstPix.width, dstPix.height);
-		m_capImage = new QImage(dstPix.width, dstPix.height, dstFmt);
-		m_capImage->fill(0);
-		m_capture->setImage(*m_capImage, "No frame");
-		m_capture->show();
+		v4l2_format copy = m_capSrcFormat;
+		v4lconvert_try_format(m_convertData, &m_capDestFormat, &m_capSrcFormat);
+		// v4lconvert_try_format sometimes modifies the source format if it thinks
+		// that there is a better format available. Restore our selected source
+		// format since we do not want that happening.
+		m_capSrcFormat = copy;
+		width = dstPix.width;
+		height = dstPix.height;
+		pixfmt = dstPix.pixelformat;
 	}
 
+	// Ensure that the initial image is large enough for native 32 bit per pixel formats
+	if (pixfmt == V4L2_PIX_FMT_RGB32 || pixfmt == V4L2_PIX_FMT_BGR32)
+		dstFmt = QImage::Format_ARGB32;
+	m_capImage = new QImage(width, height, dstFmt);
+	m_capImage->fill(0);
+	
+	updatePixelAspectRatio();
+	
+	m_capture->setFrame(m_capImage->width(), m_capImage->height(),
+			    pixfmt, m_capImage->bits(), NULL, "No frame");
+	m_capture->resize(width, height);
+	if (showFrames())
+		m_capture->show();
+
 	statusBar()->showMessage("No frame");
 	if (startCapture(srcPix.sizeimage)) {
 		m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
@@ -777,6 +1115,7 @@ void ApplicationWindow::capStart(bool start)
 
 void ApplicationWindow::closeDevice()
 {
+	stopAudio();
 	delete m_sigMapper;
 	m_sigMapper = NULL;
 	m_capStartAct->setEnabled(false);
@@ -888,8 +1227,24 @@ void ApplicationWindow::saveRaw(bool checked)
 
 void ApplicationWindow::about()
 {
+#ifdef HAVE_ALSA
+	bool alsa = true;
+#else
+	bool alsa = false;
+#endif
+#ifdef HAVE_QTGL
+	bool gl = true;
+#else
+	bool gl = false;
+#endif
+
 	QMessageBox::about(this, "V4L2 Test Bench",
-			"This program allows easy experimenting with video4linux devices.");
+			   QString("This program allows easy experimenting with video4linux devices.\n"
+				   "v. %1\n\nALSA support : %2\nOpenGL support : %3")
+			   .arg(V4L_UTILS_VERSION)
+			   .arg(alsa ? "Present" : "Not Available")
+			   .arg(gl ? "Present" : "Not Available")
+			   );
 }
 
 void ApplicationWindow::error(const QString &error)
@@ -936,33 +1291,156 @@ void ApplicationWindow::closeEvent(QCloseEvent *event)
 
 ApplicationWindow *g_mw;
 
+static void usage()
+{
+	printf("  Usage:\n"
+	       "  qv4l2 [-R] [-h] [-d <dev>] [-r <dev>] [-V <dev>]\n"
+	       "\n  -d, --device=<dev> use device <dev> as the video device\n"
+	       "                     if <dev> is a number, then /dev/video<dev> is used\n"
+	       "  -V, --vbi-device=<dev> use device <dev> as the vbi device\n"
+	       "                     if <dev> is a number, then /dev/vbi<dev> is used\n"
+	       "  -r, --radio-device=<dev> use device <dev> as the radio device\n"
+	       "                     if <dev> is a number, then /dev/radio<dev> is used\n"
+	       "  -S, --sdr-device=<dev> use device <dev> as the SDR device\n"
+	       "                     if <dev> is a number, then /dev/swradio<dev> is used\n"
+	       "  -h, --help         display this help message\n"
+	       "  -R, --raw          open device in raw mode.\n");
+}
+
+static void usageError(const char *msg)
+{
+	printf("Missing parameter for %s\n", msg);
+	usage();
+}
+
+static QString getDeviceName(QString dev, QString &name)
+{
+	bool ok;
+	name.toInt(&ok);
+	return ok ? QString("%1%2").arg(dev).arg(name) : name;
+}
+
 int main(int argc, char **argv)
 {
 	QApplication a(argc, argv);
-	QString device = "/dev/video0";
 	bool raw = false;
-	bool help = false;
-	int i;
+	QString device;
+	QString video_device;
+	QString vbi_device;
+	QString radio_device;
+	QString sdr_device;
 
 	a.setWindowIcon(QIcon(":/qv4l2.png"));
 	g_mw = new ApplicationWindow();
 	g_mw->setWindowTitle("V4L2 Test Bench");
-	for (i = 1; i < argc; i++) {
-		const char *arg = a.argv()[i];
 
-		if (!strcmp(arg, "-r"))
+	QStringList args = a.arguments();
+	for (int i = 1; i < args.size(); i++) {
+		if (args[i] == "-d" || args[i] == "--device") {
+			++i;
+			if (i >= args.size()) {
+				usageError("-d");
+				return 0;
+			}
+
+			video_device = args[i];
+			if (video_device.startsWith("-")) {
+				usageError("-d");
+				return 0;
+			}
+		} else if (args[i] == "-V" || args[i] == "--vbi-device") {
+			++i;
+			if (i >= args.size()) {
+				usageError("-V");
+				return 0;
+			}
+
+			vbi_device = args[i];
+			if (vbi_device.startsWith("-")) {
+				usageError("-V");
+				return 0;
+			}
+		} else if (args[i] == "-r" || args[i] == "--radio-device") {
+			++i;
+			if (i >= args.size()) {
+				usageError("-r");
+				return 0;
+			}
+
+			radio_device = args[i];
+			if (radio_device.startsWith("-")) {
+				usageError("-r");
+				return 0;
+			}
+		} else if (args[i] == "-S" || args[i] == "--sdr-device") {
+			++i;
+			if (i >= args.size()) {
+				usageError("-S");
+				return 0;
+			}
+
+			sdr_device = args[i];
+			if (sdr_device.startsWith("-")) {
+				usageError("-S");
+				return 0;
+			}
+		} else if (args[i].startsWith("--device")) {
+			QStringList param = args[i].split("=");
+			if (param.size() == 2) {
+				video_device = param[1];
+			} else {
+				usageError("--device");
+				return 0;
+			}
+		} else if (args[i].startsWith("--vbi-device")) {
+			QStringList param = args[i].split("=");
+			if (param.size() == 2) {
+				vbi_device = param[1];
+			} else {
+				usageError("--vbi-device");
+				return 0;
+			}
+		} else if (args[i].startsWith("--radio-device")) {
+			QStringList param = args[i].split("=");
+			if (param.size() == 2) {
+				radio_device = param[1];
+			} else {
+				usageError("--radio-device");
+				return 0;
+			}
+		} else if (args[i].startsWith("--sdr-device")) {
+			QStringList param = args[i].split("=");
+			if (param.size() == 2) {
+				sdr_device = param[1];
+			} else {
+				usageError("--sdr-device");
+				return 0;
+			}
+		} else if (args[i] == "-h" || args[i] == "--help") {
+			usage();
+			return 0;
+
+		} else if (args[i] == "-R" || args[i] == "--raw") {
 			raw = true;
-		else if (!strcmp(arg, "-h"))
-			help = true;
-		else if (arg[0] != '-')
-			device = arg;
-	}
-	if (help) {
-		printf("qv4l2 [-r] [-h] [device node]\n\n"
-		       "-h\tthis help message\n"
-		       "-r\topen device node in raw mode\n");
-		return 0;
+
+
+		} else {
+			printf("Invalid argument %s\n", args[i].toAscii().data());
+			return 0;
+		}
 	}
+
+	if (video_device != NULL)
+		device = getDeviceName("/dev/video", video_device);
+	else if (vbi_device != NULL)
+		device = getDeviceName("/dev/vbi", vbi_device);
+	else if (radio_device != NULL)
+		device = getDeviceName("/dev/radio", radio_device);
+	else if (sdr_device != NULL)
+		device = getDeviceName("/dev/swradio", sdr_device);
+	else
+		device = "/dev/video0";
+
 	g_mw->setDevice(device, raw);
 	g_mw->show();
 	a.connect(&a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()));
