@@ -141,19 +141,18 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 		qmenu.id = qctrl.id;
 		qmenu.index = qctrl.minimum;
 		ret = doioctl(node, VIDIOC_QUERYMENU, &qmenu);
-		if (ret != EINVAL)
+		if (ret != EINVAL && ret != ENOTTY)
 			return fail("can do querymenu on a non-menu control\n");
 		return 0;
 	}
-	if (qctrl.maximum >= 32)
-		return fail("currently more than 32 menu items are not supported\n");
+	bool have_default_value = false;
 	for (i = 0; i <= qctrl.maximum + 1; i++) {
 		memset(&qmenu, 0xff, sizeof(qmenu));
 		qmenu.id = qctrl.id;
 		qmenu.index = i;
 		ret = doioctl(node, VIDIOC_QUERYMENU, &qmenu);
 		if (ret && ret != EINVAL)
-			return fail("invalid QUERYMENU return code\n");
+			return fail("invalid QUERYMENU return code (%d)\n", ret);
 		if (ret)
 			continue;
 		if (i < qctrl.minimum || i > qctrl.maximum)
@@ -165,12 +164,14 @@ static int checkQCtrl(struct node *node, struct test_queryctrl &qctrl)
 			return fail("invalid menu name\n");
 		if (qmenu.reserved)
 			return fail("reserved is non-zero\n");
-		qctrl.menu_mask |= 1 << i;
+		if (i == qctrl.default_value)
+			have_default_value = true;
+		if (i < 64)
+			qctrl.menu_mask |= 1ULL << i;
 	}
 	if (qctrl.menu_mask == 0)
 		return fail("no menu items found\n");
-	if (!(qctrl.menu_mask & (1 << qctrl.default_value)))
-		return fail("the default_value is an invalid menu item\n");
+	fail_on_test(!have_default_value);
 	return 0;
 }
 
@@ -194,7 +195,7 @@ int testQueryControls(struct node *node)
 		if (ret == ENOTTY)
 			return ret;
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code\n");
+			return fail("invalid queryctrl return code (%d)\n", ret);
 		if (ret)
 			break;
 		if (checkQCtrl(node, qctrl))
@@ -244,7 +245,7 @@ int testQueryControls(struct node *node)
 		qctrl.id = id;
 		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code\n");
+			return fail("invalid queryctrl return code (%d)\n", ret);
 		if (ret)
 			continue;
 		if (qctrl.id != id)
@@ -260,7 +261,7 @@ int testQueryControls(struct node *node)
 		qctrl.id = id;
 		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
 		if (ret && ret != EINVAL)
-			return fail("invalid queryctrl return code\n");
+			return fail("invalid queryctrl return code (%d)\n", ret);
 		if (ret)
 			break;
 		if (qctrl.id != id)
@@ -352,7 +353,7 @@ int testSimpleControls(struct node *node)
 			ctrl.id = iter->id;
 			ctrl.value = iter->default_value;
 		} else if (ret)
-			return fail("g_ctrl returned an error\n");
+			return fail("g_ctrl returned an error (%d)\n", ret);
 		else if (checkSimpleCtrl(ctrl, *iter))
 			return fail("invalid control %08x\n", iter->id);
 		
@@ -405,9 +406,15 @@ int testSimpleControls(struct node *node)
 		}
 
 		if (iter->type == V4L2_CTRL_TYPE_MENU || iter->type == V4L2_CTRL_TYPE_INTEGER_MENU) {
+			int max = iter->maximum;
+
+			/* Currently menu_mask is a 64-bit value, so we can't reliably
+			 * test for menu item > 63. */
+			if (max > 63)
+				max = 63;
 			// check menu items
-			for (i = iter->minimum; i <= iter->maximum; i++) {
-				unsigned valid = iter->menu_mask & (1 << i);
+			for (i = iter->minimum; i <= max; i++) {
+				bool valid = iter->menu_mask & (1ULL << i);
 
 				ctrl.id = iter->id; 
 				ctrl.value = i;
@@ -417,7 +424,7 @@ int testSimpleControls(struct node *node)
 				if (!valid && !ret)
 					return fail("could set invalid menu item %d\n", i);
 				if (ret && ret != EINVAL)
-					return fail("setting invalid menu item returned wrong error\n");
+					return fail("setting invalid menu item returned wrong error (%d)\n", ret);
 			}
 		} else {
 			// at least min, max and default values should work
@@ -581,7 +588,7 @@ int testExtendedControls(struct node *node)
 			if (ctrls.error_idx != 0)
 				return fail("invalid error index read only control\n");
 		} else if (ret) {
-			return fail("try_ext_ctrls returned an error\n");
+			return fail("try_ext_ctrls returned an error (%d)\n", ret);
 		}
 		
 		// Try to set the current value (or the default value for write only controls)
@@ -597,7 +604,7 @@ int testExtendedControls(struct node *node)
 				ret = 0;
 			}
 			if (ret)
-				return fail("s_ext_ctrls returned an error\n");
+				return fail("s_ext_ctrls returned an error (%d)\n", ret);
 		
 			if (checkExtendedCtrl(ctrl, *iter))
 				return fail("s_ext_ctrls returned invalid control contents (%08x)\n", iter->id);
@@ -614,7 +621,7 @@ int testExtendedControls(struct node *node)
 	if (ret != EINVAL)
 		return fail("g_ext_ctrls accepted invalid control ID\n");
 	if (ctrls.error_idx != ctrls.count)
-		return fail("g_ext_ctrls(0) invalid error_idx\n");
+		return fail("g_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
 	ctrl.id = 0;
 	ctrl.size = 0;
 	ctrl.value = 0;
@@ -622,12 +629,12 @@ int testExtendedControls(struct node *node)
 	if (ret != EINVAL)
 		return fail("try_ext_ctrls accepted invalid control ID\n");
 	if (ctrls.error_idx != 0)
-		return fail("try_ext_ctrls(0) invalid error_idx\n");
+		return fail("try_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
 	ret = doioctl(node, VIDIOC_S_EXT_CTRLS, &ctrls);
 	if (ret != EINVAL)
 		return fail("s_ext_ctrls accepted invalid control ID\n");
 	if (ctrls.error_idx != ctrls.count)
-		return fail("s_ext_ctrls(0) invalid error_idx\n");
+		return fail("s_ext_ctrls(0) invalid error_idx %u\n", ctrls.error_idx);
 
 	for (iter = node->controls.begin(); iter != node->controls.end(); ++iter) {
 		struct v4l2_ext_control ctrl;
@@ -713,8 +720,8 @@ int testControlEvents(struct node *node)
 			return fail("subscribe event for control '%s' failed\n", iter->name);
 		//if (iter->type == V4L2_CTRL_TYPE_CTRL_CLASS)
 		FD_ZERO(&set);
-		FD_SET(node->fd, &set);
-		ret = select(node->fd + 1, NULL, NULL, &set, &timeout);
+		FD_SET(node->vfd.fd, &set);
+		ret = select(node->vfd.fd + 1, NULL, NULL, &set, &timeout);
 		if (ret == 0) {
 			if (iter->type != V4L2_CTRL_TYPE_CTRL_CLASS)
 				return fail("failed to find event for control '%s'\n", iter->name);
