@@ -13,31 +13,21 @@
 #include <sys/time.h>
 #include <dirent.h>
 #include <math.h>
-#include <config.h>
-
-#include <linux/videodev2.h>
-#include <libv4l2.h>
-#include <string>
 
 #include "v4l2-ctl.h"
 
-static struct v4l2_format vfmt;	/* set_format/get_format for video */
 static struct v4l2_frmsizeenum frmsize; /* list frame sizes */
 static struct v4l2_frmivalenum frmival; /* list frame intervals */
 static unsigned set_fmts;
+static __u32 width, height, pixfmt, field;
+static __u32 bytesperline[VIDEO_MAX_PLANES];
 
 void vidcap_usage(void)
 {
 	printf("\nVideo Capture Formats options:\n"
 	       "  --list-formats     display supported video formats [VIDIOC_ENUM_FMT]\n"
-	       "  --list-formats-mplane\n"
- 	       "                     display supported video multi-planar formats\n"
- 	       "                     [VIDIOC_ENUM_FMT]\n"
 	       "  --list-formats-ext display supported video formats including frame sizes\n"
 	       "                     and intervals\n"
-	       "  --list-formats-ext-mplane\n"
-	       "                     display supported video multi-planar formats including\n"
-	       "                     frame sizes and intervals\n"
 	       "  --list-framesizes=<f>\n"
 	       "                     list supported framesizes for pixelformat <f>\n"
 	       "                     [VIDIOC_ENUM_FRAMESIZES]\n"
@@ -46,25 +36,18 @@ void vidcap_usage(void)
 	       "                     list supported frame intervals for pixelformat <f> and\n"
 	       "                     the given width and height [VIDIOC_ENUM_FRAMEINTERVALS]\n"
 	       "                     pixelformat is the fourcc value as a string\n"
+	       "  --list-fields      list supported fields for the current format\n"
 	       "  -V, --get-fmt-video\n"
 	       "     		     query the video capture format [VIDIOC_G_FMT]\n"
-	       "  -v, --set-fmt-video=width=<w>,height=<h>,pixelformat=<f>\n"
-	       "                     set the video capture format [VIDIOC_S_FMT]\n"
+	       "  -v, --set-fmt-video\n"
+	       "  --try-fmt-video=width=<w>,height=<h>,pixelformat=<pf>,field=<f>,bytesperline=<bpl>\n"
+	       "                     set/try the video capture format [VIDIOC_S/TRY_FMT]\n"
 	       "                     pixelformat is either the format index as reported by\n"
-	       "                     --list-formats, or the fourcc value as a string\n"
-	       "  --try-fmt-video=width=<w>,height=<h>,pixelformat=<f>\n"
-	       "                     try the video capture format [VIDIOC_TRY_FMT]\n"
-	       "                     pixelformat is either the format index as reported by\n"
-	       "                     --list-formats, or the fourcc value as a string\n"
-	       "  --get-fmt-video-mplane\n"
-	       "     		     query the video capture format through the multi-planar API\n"
-	       "                     [VIDIOC_G_FMT]\n"
-	       "  --set-fmt-video-mplane\n"
-	       "  --try-fmt-video-mplane=width=<w>,height=<h>,pixelformat=<f>\n"
-	       "                     set/try the video capture format using the multi-planar API\n"
-	       "                     [VIDIOC_S/TRY_FMT]\n"
-	       "                     pixelformat is either the format index as reported by\n"
-	       "                     --list-formats-mplane, or the fourcc value as a string\n"
+	       "                     --list-formats, or the fourcc value as a string.\n"
+	       "                     The bytesperline option can be used multiple times, once for each plane.\n"
+	       "                     <f> can be one of:\n"
+	       "                     any, none, top, bottom, interlaced, seq_tb, seq_bt,\n"
+	       "                     alternate, interlaced_tb, interlaced_bt\n"
 	       );
 }
 
@@ -131,7 +114,7 @@ static void print_frmival(const struct v4l2_frmivalenum &frmival, const char *pr
 	}
 }
 
-static void print_video_formats_ext(int fd, enum v4l2_buf_type type)
+static void print_video_formats_ext(int fd, __u32 type)
 {
 	struct v4l2_fmtdesc fmt;
 	struct v4l2_frmsizeenum frmsize;
@@ -168,34 +151,48 @@ static void print_video_formats_ext(int fd, enum v4l2_buf_type type)
 	}
 }
 
+static void print_video_fields(int fd)
+{
+	struct v4l2_format fmt;
+	struct v4l2_format tmp;
+
+	fmt.type = vidcap_buftype;
+	if (test_ioctl(fd, VIDIOC_G_FMT, &fmt) < 0)
+		return;
+
+	printf("Supported Video Fields:\n");
+	for (__u32 f = V4L2_FIELD_NONE; f <= V4L2_FIELD_INTERLACED_BT; f++) {
+		bool ok;
+
+		tmp = fmt;
+		if (is_multiplanar)
+			tmp.fmt.pix_mp.field = f;
+		else
+			tmp.fmt.pix.field = f;
+		if (test_ioctl(fd, VIDIOC_TRY_FMT, &tmp) < 0)
+			continue;
+		if (is_multiplanar)
+			ok = tmp.fmt.pix_mp.field == f;
+		else
+			ok = tmp.fmt.pix.field == f;
+		if (ok)
+			printf("\t%s\n", field2s(f).c_str());
+	}
+}
+
 void vidcap_cmd(int ch, char *optarg)
 {
-	__u32 width, height, pixfmt;
+	__u32 colorspace;
 	char *value, *subs;
 
 	switch (ch) {
-	case OptSetVideoMplaneFormat:
-	case OptTryVideoMplaneFormat:
-		set_fmts = parse_fmt(optarg, width, height, pixfmt);
-		if (!set_fmts) {
-			vidcap_usage();
-			exit(1);
-		}
-		vfmt.fmt.pix_mp.width = width;
-		vfmt.fmt.pix_mp.height = height;
-		vfmt.fmt.pix_mp.pixelformat = pixfmt;
-		break;
-
 	case OptSetVideoFormat:
 	case OptTryVideoFormat:
-		set_fmts = parse_fmt(optarg, width, height, pixfmt);
-		if (!set_fmts) {
+		set_fmts = parse_fmt(optarg, width, height, pixfmt, field, colorspace, bytesperline);
+		if (!set_fmts || (set_fmts & FmtColorspace)) {
 			vidcap_usage();
 			exit(1);
 		}
-		vfmt.fmt.pix.width = width;
-		vfmt.fmt.pix.height = height;
-		vfmt.fmt.pix.pixelformat = pixfmt;
 		break;
 	case OptListFrameSizes:
 		if (strlen(optarg) == 4)
@@ -243,63 +240,68 @@ void vidcap_set(int fd)
 	int ret;
 
 	if (options[OptSetVideoFormat] || options[OptTryVideoFormat]) {
-		struct v4l2_format in_vfmt;
+		struct v4l2_format vfmt;
 
-		in_vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-		if (doioctl(fd, VIDIOC_G_FMT, &in_vfmt) == 0) {
-			if (set_fmts & FmtWidth)
-				in_vfmt.fmt.pix.width = vfmt.fmt.pix.width;
-			if (set_fmts & FmtHeight)
-				in_vfmt.fmt.pix.height = vfmt.fmt.pix.height;
-			if (set_fmts & FmtPixelFormat) {
-				in_vfmt.fmt.pix.pixelformat = vfmt.fmt.pix.pixelformat;
-				if (in_vfmt.fmt.pix.pixelformat < 256) {
-					in_vfmt.fmt.pix.pixelformat =
-						find_pixel_format(fd, in_vfmt.fmt.pix.pixelformat,
-								  false, false);
+		vfmt.type = vidcap_buftype;
+
+		if (doioctl(fd, VIDIOC_G_FMT, &vfmt) == 0) {
+			if (is_multiplanar) {
+				if (set_fmts & FmtWidth)
+					vfmt.fmt.pix_mp.width = width;
+				if (set_fmts & FmtHeight)
+					vfmt.fmt.pix_mp.height = height;
+				if (set_fmts & FmtPixelFormat) {
+					vfmt.fmt.pix_mp.pixelformat = pixfmt;
+					if (vfmt.fmt.pix_mp.pixelformat < 256) {
+						vfmt.fmt.pix_mp.pixelformat =
+							find_pixel_format(fd, vfmt.fmt.pix_mp.pixelformat,
+									false, true);
+					}
+				}
+				if (set_fmts & FmtField)
+					vfmt.fmt.pix_mp.field = field;
+				if (set_fmts & FmtBytesPerLine) {
+					for (unsigned i = 0; i < VIDEO_MAX_PLANES; i++)
+						vfmt.fmt.pix_mp.plane_fmt[i].bytesperline =
+							bytesperline[i];
+				} else {
+					/* G_FMT might return bytesperline values > width,
+					 * reset them to 0 to force the driver to update them
+					 * to the closest value for the new width. */
+					for (unsigned i = 0; i < vfmt.fmt.pix_mp.num_planes; i++)
+						vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
+				}
+			} else {
+				if (set_fmts & FmtWidth)
+					vfmt.fmt.pix.width = width;
+				if (set_fmts & FmtHeight)
+					vfmt.fmt.pix.height = height;
+				if (set_fmts & FmtPixelFormat) {
+					vfmt.fmt.pix.pixelformat = pixfmt;
+					if (vfmt.fmt.pix.pixelformat < 256) {
+						vfmt.fmt.pix.pixelformat =
+							find_pixel_format(fd, vfmt.fmt.pix.pixelformat,
+									false, false);
+					}
+				}
+				if (set_fmts & FmtField)
+					vfmt.fmt.pix.field = field;
+				if (set_fmts & FmtBytesPerLine) {
+					vfmt.fmt.pix.bytesperline = bytesperline[0];
+				} else {
+					/* G_FMT might return a bytesperline value > width,
+					 * reset this to 0 to force the driver to update it
+					 * to the closest value for the new width. */
+					vfmt.fmt.pix.bytesperline = 0;
 				}
 			}
-			/* G_FMT might return a bytesperline value > width,
-			 * reset this to 0 to force the driver to update it
-			 * to the closest value for the new width. */
-			in_vfmt.fmt.pix.bytesperline = 0;
+
 			if (options[OptSetVideoFormat])
-				ret = doioctl(fd, VIDIOC_S_FMT, &in_vfmt);
+				ret = doioctl(fd, VIDIOC_S_FMT, &vfmt);
 			else
-				ret = doioctl(fd, VIDIOC_TRY_FMT, &in_vfmt);
+				ret = doioctl(fd, VIDIOC_TRY_FMT, &vfmt);
 			if (ret == 0 && (verbose || options[OptTryVideoFormat]))
-				printfmt(in_vfmt);
-		}
-	}
-
-	if (options[OptSetVideoMplaneFormat] || options[OptTryVideoMplaneFormat]) {
-		struct v4l2_format in_vfmt;
-
-		in_vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
-		if (doioctl(fd, VIDIOC_G_FMT, &in_vfmt) == 0) {
-			if (set_fmts & FmtWidth)
-				in_vfmt.fmt.pix_mp.width = vfmt.fmt.pix_mp.width;
-			if (set_fmts & FmtHeight)
-				in_vfmt.fmt.pix_mp.height = vfmt.fmt.pix_mp.height;
-			if (set_fmts & FmtPixelFormat) {
-				in_vfmt.fmt.pix_mp.pixelformat = vfmt.fmt.pix_mp.pixelformat;
-				if (in_vfmt.fmt.pix_mp.pixelformat < 256) {
-					in_vfmt.fmt.pix_mp.pixelformat =
-						find_pixel_format(fd, in_vfmt.fmt.pix_mp.pixelformat,
-								  false, true);
-				}
-			}
-			/* G_FMT might return bytesperline values > width,
-			 * reset them to 0 to force the driver to update them
-			 * to the closest value for the new width. */
-			for (unsigned i = 0; i < in_vfmt.fmt.pix_mp.num_planes; i++)
-				in_vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
-			if (options[OptSetVideoMplaneFormat])
-				ret = doioctl(fd, VIDIOC_S_FMT, &in_vfmt);
-			else
-				ret = doioctl(fd, VIDIOC_TRY_FMT, &in_vfmt);
-			if (ret == 0 && (verbose || options[OptTryVideoMplaneFormat]))
-				printfmt(in_vfmt);
+				printfmt(vfmt);
 		}
 	}
 }
@@ -307,13 +309,9 @@ void vidcap_set(int fd)
 void vidcap_get(int fd)
 {
 	if (options[OptGetVideoFormat]) {
-		vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-		if (doioctl(fd, VIDIOC_G_FMT, &vfmt) == 0)
-			printfmt(vfmt);
-	}
+		struct v4l2_format vfmt;
 
-	if (options[OptGetVideoMplaneFormat]) {
-		vfmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE;
+		vfmt.type = vidcap_buftype;
 		if (doioctl(fd, VIDIOC_G_FMT, &vfmt) == 0)
 			printfmt(vfmt);
 	}
@@ -323,22 +321,16 @@ void vidcap_list(int fd)
 {
 	if (options[OptListFormats]) {
 		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats(fd, V4L2_BUF_TYPE_VIDEO_CAPTURE);
-	}
-
-	if (options[OptListMplaneFormats]) {
-		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats(fd, V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE);
+		print_video_formats(fd, vidcap_buftype);
 	}
 
 	if (options[OptListFormatsExt]) {
 		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats_ext(fd, V4L2_BUF_TYPE_VIDEO_CAPTURE);
+		print_video_formats_ext(fd, vidcap_buftype);
 	}
 
-	if (options[OptListMplaneFormatsExt]) {
-		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats_ext(fd, V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE);
+	if (options[OptListFields]) {
+		print_video_fields(fd);
 	}
 
 	if (options[OptListFrameSizes]) {
